﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;

internal static class PonyLiberator
{
    private static byte[] _addressBytes;
    private static readonly Socket _dnsSocket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
    private static readonly Socket _httpSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

    private static void Error(string msg)
    {
        Console.WriteLine("Error: {0}", msg);
        Console.WriteLine();
        Console.WriteLine("Press any key to exit.");
        Console.ReadKey();
        System.Environment.Exit(0);
    }

    private static void DnsMain()
    {
        var openDnsEndPoint = new IPEndPoint(IPAddress.Parse("208.67.222.222"), 53);
        var pendingDnsRequests = new Dictionary<short, EndPoint>();

        while (true)
        {
            try
            {
                var buffer = new byte[1024];
                EndPoint endPoint = new IPEndPoint(0, 0);

                var bytesRead = _dnsSocket.ReceiveFrom(buffer, ref endPoint);

                var requestId = BitConverter.ToInt16(buffer, 0);

                if (!endPoint.Equals(openDnsEndPoint))
                {
                    pendingDnsRequests[requestId] = endPoint;

                    // Forward request to OpenDNS.
                    try { _dnsSocket.SendTo(buffer, bytesRead, SocketFlags.None, openDnsEndPoint); }
                    catch (SocketException) {}
                }
                else if (pendingDnsRequests.TryGetValue(requestId, out endPoint))
                {
                    pendingDnsRequests.Remove(requestId);

                    var bstr = Encoding.ASCII.GetString(buffer, 0, bytesRead);
                    if (bstr.Contains("vshop") && bstr.Contains("mdc") && bstr.Contains("gameloft") && bstr.Contains("com"))
                    {
                        Console.WriteLine("vshop DNS request recieved from {0}", endPoint);

                        // Replace 208.71.186.73 with our IP address.
                        buffer[bytesRead - 4] = _addressBytes[0];
                        buffer[bytesRead - 3] = _addressBytes[1];
                        buffer[bytesRead - 2] = _addressBytes[2];
                        buffer[bytesRead - 1] = _addressBytes[3];
                    }

                    // Forward response to client.
                    try { _dnsSocket.SendTo(buffer, bytesRead, SocketFlags.None, endPoint); }
                    catch (SocketException) {}
                }
            }
            catch (Exception ex) { Console.WriteLine("Exception in DNS thread. {0}", ex); } // Pokemon exceptions, gotta catch em all!
        }
    }

    private static void HttpMain()
    {
        while (true)
        {
            try
            {
                var httpClient = _httpSocket.Accept();

                Console.WriteLine("HTTP request recieved from {0}", httpClient.RemoteEndPoint);

                var file = System.IO.File.ReadAllBytes("item_list.xml");

                var enc = Encoding.UTF8;
                httpClient.Send(enc.GetBytes("HTTP/1.0 200 OK\r\n"));
                httpClient.Send(enc.GetBytes("Content-Type: application/xml; charset=utf-8\r\n"));
                httpClient.Send(enc.GetBytes(string.Format("Content-Length:{0}\r\n\r\n", file.Length)));
                httpClient.Send(file);
            }
            catch (Exception ex) { Console.WriteLine("Exception in HTTP thread. {0}", ex); } // Pokemon exceptions, gotta catch em all!
        }
    }

    private static void Main()
    {
        Console.WriteLine("PonyLiberator Version 1");
        Console.WriteLine();

        var addresses = Dns.GetHostEntry(Dns.GetHostName()).AddressList.Where(a => a.AddressFamily == AddressFamily.InterNetwork).ToArray();
        IPAddress address;

        if (addresses.Length == 0)
            Error("No IP addresses found.");

        if (addresses.Length == 1)
            address = addresses[0];

        else
        {
            Console.WriteLine("Multiple IP addresses found:");
            for (var i = 0; i < addresses.Length; i++)
                Console.WriteLine("{0}: {1}", i, addresses[i]);
            Console.WriteLine();

            Console.Write("Which one do you want to use? (select using 0-{0}) > ", addresses.Length - 1);

            while (true)
            {
                var i = Console.ReadKey().KeyChar - '0';

                if (i < 0 || i > addresses.Length - 1)
                {
                    Console.Beep();
                    Console.CursorLeft--;
                    continue;
                }

                Console.WriteLine();
                Console.WriteLine();

                address = addresses[i];

                break;
            }
        }

        _addressBytes = address.GetAddressBytes();
        Console.WriteLine("Using {0}", address);
        Console.WriteLine();

        try { _dnsSocket.Bind(new IPEndPoint(address, 53)); }
        catch { Error("Can't bind to port 53.  Is something else already using it?"); }

        try
        {
            _httpSocket.Bind(new IPEndPoint(address, 80));
            _httpSocket.Listen(10);
        }
        catch { Error("Can't open port 80.  Is something else already using it?"); }

        new Thread(DnsMain).Start();
        new Thread(HttpMain).Start();

        Console.WriteLine("Ready");

        Console.ReadLine();
        System.Environment.Exit(0);
    }
}